/**
 * \file
 *
 * \brief Empty user application template
 *
 */

/**
 * \mainpage User Application template doxygen documentation
 *
 * \par Empty user application template
 *
 * This is a bare minimum user application template.
 *
 * For documentation of the board, go \ref group_common_boards "here" for a link
 * to the board-specific documentation.
 *
 * \par Content
 *
 * -# Include the ASF header files (through asf.h)
 * -# Minimal main function that starts with a call to board_init()
 * -# Basic usage of on-board LED and button
 * -# "Insert application code here" comment
 *
 */

/*
 * Include header files for all drivers that have been imported from
 * Atmel Software Framework (ASF).
 */
/*
 * Support and FAQ: visit <a href="http://www.atmel.com/design-support/">Atmel Support</a>
 */
#include <asf.h>

#define INCREASE 0
#define DECREASE 1

//Port I/O
#define TC0_TIOA_CH1 IOPORT_CREATE_PIN(PIOA, 15)

//Timer configuration
#define TC0_CHANNEL1 1

//DAC configuration
#define HALFWORD_MODE 0
#define MAXSPEED_MODE_DISABLED 0
#define STARTUP_TIME_1920_TICKS 30
#define DACC_CHANNEL0 0
#define TC_CHANNEL1_TIO 2
#define TXRDY 0x01

int main (void)
{
	uint16_t n;
	bool Increase_or_Decrease = INCREASE;
	
	//clock configuration and initialization
	sysclk_init();
	
	/*Disable the watchdog timer and configure/initialize
	port pins connected to various components incorporated 
	into the SAM4S Xplained development platform, e.g., the 
	NAND flash, the OLED interface, the LEDs, the SW0 pushbutton.*/
	board_init();
	
	//enable peripheral clock for TC module 0 channel 1
	pmc_enable_periph_clk(ID_TC1);
	
	//enable peripheral clock for DACC
	pmc_enable_periph_clk(ID_DACC);
	
	//connect peripheral B to pin A15
	pio_configure_pin(TC0_TIOA_CH1, PIO_TYPE_PIO_PERIPH_B);
	
	tc_init(TC0, TC0_CHANNEL1, 
							TC_CMR_TCCLKS_TIMER_CLOCK2	//MCK/8
						|	TC_CMR_WAVE	//waveform mode
						|	TC_CMR_WAVSEL_UP_RC	//count up, reset on RC match
						|	TC_CMR_ACPC_TOGGLE	//toggle TIOA on RC match
						);
	
	//load RC with 6 to obtain a 1 MHz square wave on TIOA
	tc_write_rc(TC0, TC0_CHANNEL1, 6);
	
	//begin DACC configuration by resetting the DACC hardware
	dacc_reset(DACC);
	
	//write one 16-bit value at a time, not two 16-bit values in one 32-bit word
	dacc_set_transfer_mode(DACC, HALFWORD_MODE);
		
	//refer to the article for details
	dacc_set_timing(DACC, MAXSPEED_MODE_DISABLED, STARTUP_TIME_1920_TICKS);
	
	//select channel 0
	dacc_set_channel_selection(DACC, DACC_CHANNEL0);
	
	/*Choose the TIO output from timer/counter channel 1 as the trigger.
	Note that "channel 1" in this case refers only to channel 1 of timer/counter
	module 0, not channel 1 of timer/counter module 1. Also, the datasheet
	specifies only "TIO output" as the trigger source, without clarifying whether
	this refers to TIOA or TIOB or both. With a little experimentation I determined
	that only TIOA will trigger the DAC, not TIOB.*/
	dacc_set_trigger(DACC, TC_CHANNEL1_TIO);
	
	//enable DACC channel 0	
	dacc_enable_channel(DACC, DACC_CHANNEL0);

	//start the timer channel that generates the trigger signal	
	tc_start(TC0, TC0_CHANNEL1);
	
	n = 0;
	
	while (1)
	{
		//check the TXRDY flag
		if ( (dacc_get_interrupt_status(DACC) & TXRDY) == false) {
			continue; }
		
		//write the conversion value
		dacc_write_conversion_data(DACC, n);
		
		if (Increase_or_Decrease == INCREASE)
		{
			if (n == 4095) {
				Increase_or_Decrease = DECREASE; }
			else {
				n++; }
		}
		
		else
		{
			if (n == 0) {
				Increase_or_Decrease = INCREASE; }
			else {
				n--; }
		}
	}
}
